@@ -10,14 +10,15 @@ from typing import Any
 from typing import Callable
 from typing import ContextManager
 from typing import Dict
-from typing import Hashable
 from typing import Iterable
 from typing import Iterator
+from typing import List
 from typing import Optional
 from typing import Sequence
 from typing import Tuple
 from typing import Union
 from typing import overload
+from urllib.parse import urlsplit
 
 if sys.version_info >= (3, 8):
     from typing import Literal
@@ -32,7 +33,7 @@ from paquo.classes import QuPathPathClass
 from paquo.images import ImageProvider
 from paquo.images import QuPathImageType
 from paquo.images import QuPathProjectImageEntry
-from paquo.images import SimpleURIImageProvider
+from paquo.images import SimpleFileImageId
 from paquo.java import URI
 from paquo.java import BufferedImage
 from paquo.java import DefaultProject
@@ -90,7 +91,6 @@ class _ProjectImageEntriesProxy(collections_abc.Sequence):
         if removed:
             for key in removed:  # pragma: no cover
                 _ = self._images.pop(key)
-                raise NotImplementedError("removal not yet implemented")
 
     def __iter__(self) -> Iterator[QuPathProjectImageEntry]:
         return iter(self._images.values())
@@ -165,7 +165,7 @@ def _stash_project_files(project_dir: pathlib.Path):
     # done
 
 
-DEFAULT_IMAGE_PROVIDER: Any = SimpleURIImageProvider()
+DEFAULT_IMAGE_PROVIDER: Any = ImageProvider()
 
 
 ProjectIOMode = Literal["r", "r+", "w", "w+", "a", "a+", "x", "x+"]
@@ -274,12 +274,37 @@ class QuPathProject:
             # update the proxy
             self._image_entries_proxy.refresh()
 
+    @overload
+    def add_image(
+        self,
+        image_id: SimpleFileImageId,
+        image_type: Optional[QuPathImageType] = ...,
+        *,
+        allow_duplicates: bool = ...,
+        return_list: Literal[True],
+    ) -> List[QuPathProjectImageEntry]:
+        ...
+
+    @overload
+    def add_image(
+        self,
+        image_id: SimpleFileImageId,
+        image_type: Optional[QuPathImageType] = ...,
+        *,
+        allow_duplicates: bool = ...,
+        return_list: Literal[False] = ...,
+    ) -> Union[QuPathProjectImageEntry, List[QuPathProjectImageEntry]]:
+        ...
+
     @redirect(stderr=True, stdout=True)
-    def add_image(self,
-                  image_id: Any,  # this should actually be ID type of the image provider
-                  image_type: Optional[QuPathImageType] = None,
-                  *,
-                  allow_duplicates: bool = False) -> QuPathProjectImageEntry:
+    def add_image(
+        self,
+        image_id: SimpleFileImageId,
+        image_type: Optional[QuPathImageType] = None,
+        *,
+        allow_duplicates: bool = False,
+        return_list: bool = False,
+    ) -> Union[QuPathProjectImageEntry, List[QuPathProjectImageEntry]]:
         """add an image to the project
 
         Parameters
@@ -300,28 +325,26 @@ class QuPathProject:
         img_uri = self._image_provider.uri(image_id)
         if img_uri is None:
             raise FileNotFoundError(f"image_provider can't provide URI for requested image_id: '{image_id}'")
-        img_id = self._image_provider.id(img_uri)
-        if not img_id == image_id:  # pragma: no cover
-            _log.warning(f"image_provider roundtrip error: '{image_id}' -> uri -> '{img_id}'")
-            raise RuntimeError("the image provider failed to roundtrip the image id correctly")
+        # img_id = self._image_provider.id(img_uri)
+        # if img_id != image_id:  # pragma: no cover
+        #     _log.warning(f"image_provider roundtrip error: '{image_id}' -> uri -> '{img_id}'")
 
         if not allow_duplicates:
             for entry in self.images:
-                uri = self._image_provider.id(entry.uri)
-                if img_id == uri:
-                    raise FileExistsError(img_id)
+                if img_uri == self._image_provider.uri(entry.uri):
+                    raise FileExistsError(image_id)
 
         # first get a server builder
         try:
             support = ImageServerProvider.getPreferredUriImageSupport(
                 BufferedImage,
-                String(str(img_uri))
+                String(img_uri),
             )
         except IOException:  # pragma: no cover
             # it's possible that an image_provider returns an URI but that URI
             # is not actually reachable. In that case catch the java IOException
             # and raise a FileNotFoundError here
-            raise FileNotFoundError(image_id)
+            raise FileNotFoundError(f"{image_id!r} as {img_uri!r}")
         except ExceptionInInitializerError:
             raise OSError("no preferred support found")
         if not support:
@@ -329,53 +352,60 @@ class QuPathProject:
         server_builders = list(support.getBuilders())
         if not server_builders:
             raise OSError("no supported server builders found")  # pragma: no cover
-        server_builder = server_builders[0]
 
-        with self._stage_image_entry(server_builder) as j_entry:
-            # all of this happens in qupath.lib.gui.commands.ProjectImportImagesCommand
-            try:
-                server = server_builder.build()
-            except IOException:
-                _, _, _sb = server_builder.__class__.__name__.rpartition(".")
-                raise OSError(f"{_sb} can't open {str(image_id)}")
-            j_entry.setImageName(ServerTools.getDisplayableImageName(server))
+        entries = []
+        for server_builder in server_builders:
+            with self._stage_image_entry(server_builder) as j_entry:
+                # all of this happens in qupath.lib.gui.commands.ProjectImportImagesCommand
+                try:
+                    server = server_builder.build()
+                except IOException:
+                    _, _, _sb = server_builder.__class__.__name__.rpartition(".")
+                    raise OSError(f"{_sb} can't open {str(image_id)}")
+                j_entry.setImageName(ServerTools.getDisplayableImageName(server))
 
-            # add some informative logging
-            _md = server.getMetadata()
-            width = int(_md.getWidth())
-            height = int(_md.getHeight())
-            downsamples = [float(x) for x in _md.getPreferredDownsamplesArray()]
-            target_downsample = math.sqrt(width / 1024.0 * height / 1024.0)
-            _log.info(f"Image[{width}x{height}] with downsamples {downsamples}")
-            if not any(d >= target_downsample for d in downsamples):
-                _log.warning("No matching downsample for thumbnail! This might take a long time...")
+                # add some informative logging
+                _md = server.getMetadata()
+                width = int(_md.getWidth())
+                height = int(_md.getHeight())
+                downsamples = [float(x) for x in _md.getPreferredDownsamplesArray()]
+                target_downsample = math.sqrt(width / 1024.0 * height / 1024.0)
+                _log.info(f"Image[{width}x{height}] with downsamples {downsamples}")
+                if not any(d >= target_downsample for d in downsamples):
+                    _log.warning("No matching downsample for thumbnail! This might take a long time...")
 
-            # set the project thumbnail
-            try:
-                thumbnail = ProjectImportImagesCommand_getThumbnailRGB(server, None)
-            except NegativeArraySizeException:  # pragma: no cover
-                raise RuntimeError(
-                    "Thumbnailing FAILED. Image might be too large and has no embedded thumbnail."
-                )
-            else:
-                j_entry.setThumbnail(thumbnail)
+                # set the project thumbnail
+                try:
+                    thumbnail = ProjectImportImagesCommand_getThumbnailRGB(server, None)
+                except NegativeArraySizeException:  # pragma: no cover
+                    raise RuntimeError(
+                        "Thumbnailing FAILED. Image might be too large and has no embedded thumbnail."
+                    )
+                else:
+                    j_entry.setThumbnail(thumbnail)
 
-        py_entry = self._image_entries_proxy[-1]
-        if image_type is not None:
-            py_entry.image_type = image_type
-        # save project after adding image
-        py_entry.save()
-        self.save(images=False)
-        return py_entry
+            py_entry = self._image_entries_proxy[-1]
+            if image_type is not None:
+                py_entry.image_type = image_type
+            # save project after adding image
+            py_entry.save()
+            self.save(images=False)
+            entries.append(py_entry)
+        if return_list or len(entries) > 1:
+            return entries
+        else:
+            return entries[0]
 
-    def is_readable(self) -> Dict[Hashable, bool]:
+    def is_readable(self) -> Dict[str, bool]:
         """verify if images are reachable"""
         readability_map = {}
         for image in self.images:
-            image_id = self._image_provider.id(image.uri)
-            if image_id in readability_map:  # pragma: no cover
+            uri = image.uri
+            if uri is None:
+                raise RuntimeError(f"entry has None uri: {image!r}")
+            if uri in readability_map:  # pragma: no cover
                 raise RuntimeError("received the same image_id from image_provider for two different images")
-            readability_map[image_id] = image.is_readable()
+            readability_map[str(uri)] = image.is_readable()
         return readability_map
 
     def update_image_paths(self, *, try_relative: bool = False, **rebase_kwargs) -> None:
@@ -390,7 +420,7 @@ class QuPathProject:
             at a different location.
         **rebase_kwargs:
             keyword arguments are handed over to the image provider instance.
-            The default image provider is a paquo.images.SimpleURIImageProvider
+            The default image provider is a paquo.images.ImageProvider
             which uses the uri2uri keyword argument. (A mapping from old URI to new
             URI: Mapping[str, str])
 
@@ -432,6 +462,33 @@ class QuPathProject:
         for image in self.images:
             image.java_object.updateServerURIs(uri2uri)
 
+    @redirect(stderr=True, stdout=True)
+    def remove_image(
+        self,
+        image_entry: Union[QuPathProjectImageEntry, int],
+    ) -> None:
+        """
+        Delete an image from the QuPath project.
+
+        Parameters
+        ----------
+        image_entry:
+            the image entry to be removed
+        """
+        if isinstance(image_entry, int):
+            image_entry = self.images[image_entry]
+        if not isinstance(image_entry, QuPathProjectImageEntry):
+            raise TypeError(
+                f"expected QuPathProjectImageEntry, got: {type(image_entry).__name__!r}"
+            )
+        if self._readonly:
+            raise OSError("project in readonly mode")
+        try:
+            self.java_object.removeImage(image_entry.java_object, False)
+        finally:
+            self._image_entries_proxy.refresh()
+            self.save(images=False)
+
     @property
     def uri(self) -> str:
         """the uri identifying the project location"""
